//World.cpp - implementation of World class

#include "World.h"
#include "Organism.h"
#include "Lion.h"
#include "Ant.h"
using namespace std;


//constructor - fired by the driver
World::World()
{

	//seed the RNG
	srand ( (unsigned int)time(NULL) );

	//randomly populate the world
	//with 5 lions, 100 ants

	//organism to insert
	Organism *localOrg;

	//control placement flow
	bool lionPlaced;
	bool antPlaced;

	//variables for coordinates
    int randomX;
	int randomY;
	
	//initalize the array
	for (int i=0;i<GRID_HEIGHT;i++)
		for (int j=0;j<GRID_WIDTH;j++)
		{
			localOrg = new Organism;//just a nonce
			this->setOrganism(localOrg,i,j);
		}

	//place 5 lions
	for (int lion=0;lion<5;lion++)
	{
		localOrg = new Lion(this,GRID_WIDTH,GRID_HEIGHT);// a new lion
		lionPlaced = false;
		while (!lionPlaced)
		{
			randomX = genrand(GRID_HEIGHT);
			randomY = genrand(GRID_WIDTH);
			//see if a lion is already there
			Lion *pLion = dynamic_cast<Lion*>(this->getOrganism(randomX,randomY));
			if (((long)pLion == 0)) //it's empty (well, not a lion, anyway)
			{
				localOrg->setPosition(randomX,randomY);//set awareness of its position
				this->setOrganism(localOrg, randomX, randomY);//place it
				lionPlaced = true;
			}
		}	
	
	}

	//place 100 ants
    for (int ant=0;ant<100;ant++)
	{
		localOrg = new Ant(this,GRID_WIDTH,GRID_HEIGHT);// a new ant
		antPlaced = false;
		while (!antPlaced)
		{
			randomX = genrand(GRID_HEIGHT);
			randomY = genrand(GRID_WIDTH);
			//see if a something is already there
			Lion *pLion = dynamic_cast<Lion*>(this->getOrganism(randomX,randomY));
			Ant *pAnt = dynamic_cast<Ant*>(this->getOrganism(randomX,randomY));
			if (((long)pLion == 0) && ((long)pAnt == 0)) //it's empty
			{ 
				localOrg->setPosition(randomX,randomY);//set awareness of its position
      			this->setOrganism(localOrg, randomX, randomY);//place it
				antPlaced = true;
			}
		}	
	
	}
    
	

    

}

//virtual destructor
World::~World(){}

//execute a time step (called by the driver)
void World::run()
{

	//1. reset "already moved" flags on all critters
	//so that they're eligible to move (once)
    for (int i=0;i<GRID_HEIGHT;i++)
		for (int j=0;j<GRID_WIDTH;j++)
			this->getOrganism(i,j)->startTurn();

	//2. all of the lions "move" (may include eating)
	for (int i=0;i<GRID_HEIGHT;i++)
		for (int j=0;j<GRID_WIDTH;j++)
		{
			Lion *pLion = dynamic_cast<Lion*>(this->getOrganism(i,j));
			if ((long)pLion != 0) //it's a lion
			{
				//pLion->move();//not polymorphic
				this->getOrganism(i,j)->move();//polymorphic
			}
		}

    //3. all of the (remaining) ants move
	for (int i=0;i<GRID_HEIGHT;i++)
		for (int j=0;j<GRID_WIDTH;j++)
		{
			Ant *pAnt = dynamic_cast<Ant*>(this->getOrganism(i,j));
			if ((long)pAnt != 0)
			{
				this->getOrganism(i,j)->move();
			}
		}

    //4.lions breed
	for (int i=0;i<GRID_HEIGHT;i++)
		for (int j=0;j<GRID_WIDTH;j++)
		{
			Lion *pLion = dynamic_cast<Lion*>(this->getOrganism(i,j));
			if ((long)pLion != 0)
			{
				this->getOrganism(i,j)->breed();
			}
		}

    //5. ants breed
    for (int i=0;i<GRID_HEIGHT;i++)
		for (int j=0;j<GRID_WIDTH;j++)
		{
			Ant *pAnt = dynamic_cast<Ant*>(this->getOrganism(i,j));
			if ((long)pAnt != 0)
			{
				this->getOrganism(i,j)->breed();
			}
		}

    //6. lions starve
	for (int i=0;i<GRID_HEIGHT;i++)
		for (int j=0;j<GRID_WIDTH;j++)
		{
			Lion *pLion = dynamic_cast<Lion*>(this->getOrganism(i,j));
			if ((long)pLion != 0)
			{
				pLion->starve();
			}
		}
}

//returns a random number
//given starting point and range (exclusive)
//http://stackoverflow.com/a/9484709
//http://www.cplusplus.com/articles/EywTURfi/
int World::genrand(int range)
{
	if (range <= 0) return 0;
	else return (rand() %(range));
}

//put an organism into the world, given the organism and its coordinates
void World::setOrganism(Organism *organism, int x, int y )
{
	this->grid[x][y] = organism;
	//organism->setPosition(x,y); //did it before I called this!
}

//fetch organism from world given its coordinates
Organism *World::getOrganism( int x, int y )
{
	return this->grid[x][y];
}

//output operator (out of class)
//outputs the world grid
//called by the driver
ostream& operator<< (ostream &output, World &world)
{
	//set to "gray" so that grid isn't distracting
	SetConsoleTextAttribute(GetStdHandle( STD_OUTPUT_HANDLE ), FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
	for (int bar=0;bar<=(GRID_WIDTH*2);bar++)
			output << "-";//row separator
    output << endl;
	for (int i=0;i<GRID_HEIGHT;i++)
	{
		output << "|";//column separator
		for (int j=0;j<GRID_WIDTH;j++)
		{			
			output << world.getOrganism(i,j);//polymorphism! :-) //maybe not
			output << "|";
		}
		output << endl;
		for (int bar=0;bar<=(GRID_WIDTH*2);bar++)
			output << "-";
		output << endl;
	}
	//back to white again
	SetConsoleTextAttribute(GetStdHandle( STD_OUTPUT_HANDLE ), FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY);
	return output;
}//end friend<<